/*
 * File Archive Library
 * Copyright (C) 2000-2001 Patrizio Bruno <patrizio@dada.it>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Read the file LICENSE for more.
 *
 */

/*
 * $Log: utils.c,v $
 * Revision 1.9  2001/05/21 11:06:31  rusty
 *
 * byte ordering fixed
 *
 * Revision 1.8  2001/05/16 16:08:17  rusty
 * *** empty log message ***
 *
 * Revision 1.7  2001/05/16 11:41:53  rusty
 *
 * code cleanup
 *
 * Revision 1.6  2001/05/16 11:34:03  rusty
 *
 * minor bug fixes
 *
 * Revision 1.5  2001/05/16 11:04:57  rusty
 *
 * some debug code removed
 * added a function displaying the internal version of the library: far_version() in utils.c
 *
 * Revision 1.4  2001/05/16 10:50:39  rusty
 *
 * fixed the checksum control on archive expand
 * some little code clean up
 *
 * Revision 1.3  2000/09/21 11:27:44  rusty
 *
 * bug fixes
 *
 * Revision 1.2  2000/09/20 17:43:07  rusty
 *
 * why checksums of big files seem to be corrupted?
 *
 * Revision 1.1  2000/09/20 14:10:22  rusty
 *
 * First working (basic) release
 *
 */

#include "far.h"
#include "utils.h"

extern char *arch_version;
extern char *unarch_version;

int is_adir(const char *path)
{
	struct	stat	check;

	memset(&check, 0, sizeof(check));
	lstat(path, &check);

	return S_ISDIR(check.st_mode);
}

int far_checksum(FAR_INFO *info, unsigned char *buffer)
{
	unsigned	char	sum[16];
	md5_state_t	md5;

	md5_init(&md5);
	md5_append(&md5, buffer, info->entry_info.st_size);
	md5_finish(&md5, sum);

	return memcmp(info->chksum, sum, 16);
}

void make_list(char *string, char *delim, char *ilist[], int len)
{
	unsigned	int	finished	= 0, 
				dlen		= 0, 
				i		= 0;
	char			*ptr		= (char *)NULL;

	memset(ilist, 0, (sizeof(char*) * len + 5));
	ptr = string;
	dlen = strlen(delim);

	do
	{
		ilist[i] = ptr;

		if(((ptr = strstr(ptr, delim)) != NULL))
		{
			(*ptr) = 0;
			ptr += dlen;
			i++;
		}
		else
		{
			i++;
			ilist[i] = NULL;
			finished = 1;
		}
	} while(!finished && (i < len));
}

/* How many times does a substring appear in a string */
int strcount(char *haystack, char *needle)
{
	int	i	= 0;
	char	*ptr	= (char *)NULL;

	ptr = haystack;

	while((ptr = strstr(ptr, needle)) != NULL)
	{
		ptr += strlen(needle);
		i++;
	}

	return i;
}

char *strjoin(char **array, int n_memb, char *delim)
{
	size_t	tot_len	= 0;
	int	count	= 0;
	char	*result	= (char *)NULL;

	if(array && delim)
	{
		for(; count < n_memb ; count++)
		{
			tot_len += (strlen(array[count]) + strlen(delim));
		}

		tot_len++;
		result = calloc(1, tot_len);

		for(count = 0 ; count < n_memb ; count++)
		{
			strcat(result, array[count]);

			if(count != (n_memb - 1))
			{
				strcat(result, delim);
			}
		}
	}

	return result;
}

char *far_version(void)
{
	static	char	result[1024];

	sprintf(result, "%d.%d.%d%s%s",
		atoi(strpbrk(FAR_VERSION, "0123456789")),
		atoi(strchr(arch_version, '.') + 1),
		atoi(strchr(unarch_version, '.') + 1),
		(FARVER_BETA ? FARVER_BETA : ""),
		(FARVER_ALPHA ? FARVER_ALPHA : ""));

	return &result[0];
}

int far_printf(FILE *fp, char *fmt, ...)
{
	int	a	= 0;

	if(getenv("FAR_VERBOSE"))
	{
		va_list	ap;
		va_start(ap, fmt);
		a = vfprintf(fp, (const char *)fmt, ap);
		va_end(ap);
	}

	return a;
}

void far_byteorder_conv(FAR_INFO *info)
{
	info->entry_info.st_dev = htons(info->entry_info.st_dev);
	info->entry_info.st_ino = htonl(info->entry_info.st_ino);
	info->entry_info.st_mode = htonl(info->entry_info.st_mode);
	info->entry_info.st_nlink = htonl(info->entry_info.st_nlink);
	info->entry_info.st_uid = htonl(info->entry_info.st_uid);
	info->entry_info.st_gid = htonl(info->entry_info.st_gid);
	info->entry_info.st_rdev = htons(info->entry_info.st_rdev);
	info->entry_info.st_size = htonl(info->entry_info.st_size);
	info->entry_info.st_blksize = htonl(info->entry_info.st_blksize);
	info->entry_info.st_blocks = htonl(info->entry_info.st_blocks);
	info->entry_info.st_atime = htonl(info->entry_info.st_atime);
	info->entry_info.st_mtime = htonl(info->entry_info.st_mtime);
	info->entry_info.st_ctime = htonl(info->entry_info.st_ctime);
	info->offset = htonl(info->offset);
}

